from dataclasses import dataclass

import pyglet
from pyglet import shapes

from .settings import FONT, WIDGETS_RADIUS
from .utils import collidepoint, brighten


@dataclass
class Rect:
    x: int
    y: int
    width: int
    height: int

    @property
    def topleft(self):
        return (self.left, self.top)

    @property
    def topright(self):
        return (self.right, self.top)

    @property
    def bottomleft(self):
        return (self.left, self.bottom)

    @property
    def bottomright(self):
        return (self.right, self.bottom)

    @property
    def left(self):
        return self.x

    @property
    def right(self):
        return self.x + self.width

    @property
    def top(self):
        return self.y + self.height

    @property
    def bottom(self):
        return self.y

    @property
    def center(self):
        return self.x + self.width // 2, self.y + self.height // 2

    def contains(self, x, y):
        return collidepoint(
            self.x, self.y, self.x + self.width, self.y + self.height, x, y
        )

    def collide(self, rect):
        # return true if the two rectangles are overlapping in any way
        if rect.left >= self.right:
            return False
        if rect.right <= self.left:
            return False
        if rect.top <= self.bottom:
            return False
        if rect.bottom >= self.top:
            return False
        return True

    def copy(self):
        return Rect(self.x, self.y, self.width, self.height)


@dataclass
class HBox:
    x: int
    y: int
    height: int
    padding: int = 4

    def add(self, width):
        r = Rect(self.x, self.y, width, self.height)
        self.x += width + self.padding
        return r


@dataclass
class VBox:
    x: int
    y: int
    width: int
    padding: int = 4

    def add(self, height):
        r = Rect(self.x, self.y, self.width, height)
        self.y -= height + self.padding
        return r


@dataclass
class Style:
    text_color: tuple[int, int, int, int] = (50, 50, 50, 255)
    color: tuple[int, int, int] = (200, 200, 200)
    highlight: tuple[int, int, int] = (200, 255, 200)
    bold: bool = False


class SimpleDropdown:
    def __init__(
        self, rect, label, options, onchange=None, style=Style(), invert=False
    ):
        self.invert = invert
        self.options = options
        self.selected_index = 0
        self.expanded = False
        self.style = style
        self.label = label
        self.onchange = onchange
        self.rect = rect
        self.radius = WIDGETS_RADIUS

        # Dimensions
        self.triangle_size = int(rect.height * 0.5)

    def get_triangle(self):
        triangle_x = self.rect.x + self.rect.width - self.triangle_size - 4
        if not self.expanded:
            margin = self.rect.height - self.triangle_size
            triangle_y = (
                self.rect.y + (self.rect.height // 2) - int(0.5 * self.triangle_size)
            )
            return shapes.Triangle(
                triangle_x,
                triangle_y + self.triangle_size,
                triangle_x + self.triangle_size,
                triangle_y + self.triangle_size,
                triangle_x + self.triangle_size / 2,
                triangle_y,
                color=self.style.text_color,
            )
        else:
            margin = (self.rect.height - self.triangle_size) // 2
            triangle_y = self.rect.y + self.rect.height - margin
            return shapes.Triangle(
                triangle_x,
                triangle_y - self.triangle_size,
                triangle_x + self.triangle_size,
                triangle_y - self.triangle_size,
                triangle_x + self.triangle_size / 2,
                triangle_y,
                color=self.style.text_color,
            )

    def draw(self, cursor):
        # Dropdown box

        is_hovered = self.rect.contains(*cursor)

        color = list(self.style.color)

        if is_hovered:
            color = brighten(color)

        rect = self.rect
        diameter = 2 * self.radius

        coordinates = [
            (rect.x, rect.y + rect.height - diameter),
            (
                rect.x + rect.width - diameter,
                rect.y + rect.height - diameter,
            ),
        ]
        if not self.expanded:
            coordinates.extend(
                [
                    (rect.x, rect.y),
                    (rect.x + rect.width - diameter, rect.y),
                ]
            )

        for corner_x, corner_y in coordinates:
            shapes.Circle(
                corner_x + self.radius,
                corner_y + self.radius,
                self.radius,
                color=color,
            ).draw()

        shapes.Rectangle(
            rect.x + self.radius,
            rect.y,
            rect.width - diameter,
            rect.height,
            color=color,
        ).draw()
        shapes.Rectangle(
            rect.x,
            rect.y + self.radius,
            rect.width,
            rect.height - diameter,
            color=color,
        ).draw()
        if self.expanded:
            shapes.Rectangle(
                rect.x,
                rect.y,
                rect.width,
                self.radius,
                color=color,
            ).draw()

        if self.expanded or not self.options or self.selected_index < 0:
            text = self.label + ":"
        else:
            text = self.options[self.selected_index]["name"]

        # Selected option
        pyglet.text.Label(
            text,
            x=self.rect.x + 10,
            y=self.rect.y + self.rect.height // 2,
            anchor_x="left",
            anchor_y="center",
            color=self.style.text_color,
        ).draw()

        # Triangle button
        self.get_triangle().draw()

        x_match = self.rect.x < cursor[0] < self.rect.x + self.rect.width

        # Expanded options
        if self.expanded:
            for i, option in enumerate(self.options):
                option_x = self.rect.x
                if self.invert:
                    option_y = self.rect.y + i * self.rect.height
                else:
                    option_y = self.rect.y - (i + 1) * self.rect.height
                option_height = self.rect.height
                if x_match and option_y < cursor[1] < option_y + option_height:
                    color = self.style.highlight
                else:
                    color = self.style.color
                shapes.Rectangle(
                    option_x, option_y, self.rect.width, option_height, color=color
                ).draw()

                label = option["name"]

                pyglet.text.Label(
                    label,
                    x=option_x + 10,
                    y=option_y + option_height // 2,
                    color=self.style.text_color,
                    anchor_x="left",
                    anchor_y="center",
                    bold=i == self.selected_index,
                    font_name=FONT,
                ).draw()

    def unfocus(self):
        self.expanded = False

    def on_mouse_press(self, x, y, button, modifiers):
        # FIXME: inverted mode is probably broken
        menu_height = 0
        if self.expanded:
            menu_height = self.rect.height * (len(self.options) + 1)
        if self.invert:
            menu_height *= -1

        x_matches = self.rect.x < x < self.rect.x + self.rect.width
        if x_matches and self.rect.y - menu_height < y < self.rect.y + self.rect.height:
            old_index = self.selected_index
            # Dropdown button clicked
            if self.rect.y < y < self.rect.y + self.rect.height:
                self.expanded = not self.expanded
            else:
                # Check which option is clicked
                for i, option in enumerate(self.options):
                    if self.invert:
                        option_y = self.rect.y + (i + 1) * self.rect.height
                    else:
                        option_y = self.rect.y - (i + 1) * self.rect.height
                    if option_y < y < option_y + self.rect.height:
                        self.selected_index = i
                        self.expanded = False
                        break
            if old_index != self.selected_index:
                if self.onchange:
                    self.onchange()
            return True

    def get_value(self):
        return self.get_selected_option()["value"]

    def get_selected_option(self):
        return self.options[self.selected_index]

    def get_selected_index(self):
        return self.selected_index


class Button:
    def __init__(
        self, rect, label, action=lambda: None, style=Style(), togglable=False
    ):
        self.style = style
        self.action = action
        self.togglable = togglable
        self.toggled = False
        self.rect = rect
        self.style = style
        self.label = label
        self.radius = WIDGETS_RADIUS

    @property
    def ex(self):
        return self.rect.x + self.rect.width

    @property
    def ey(self):
        return self.rect.y + self.rect.height

    def contains(self, x, y):
        return self.rect.x < x < self.ex and self.rect.y < y < self.ey

    def unfocus(self):
        return

    def draw(self, cursor):
        # Draw rounded borders using circles and rectangles
        rect = self.rect
        style = self.style
        self.text = pyglet.text.Label(
            self.label,
            x=rect.x + rect.width // 2,
            y=rect.y + rect.height // 2,
            anchor_x="center",
            anchor_y="center",
            color=style.text_color,
            bold=style.bold,
            font_name=FONT,
        )
        contains = self.contains(*cursor)

        if self.togglable and self.toggled:
            color = list(style.highlight)
        else:
            color = list(style.color)

        if contains:
            color = brighten(color)

        # Draw rounded corners using circles
        diameter = 2 * self.radius
        coordinates = [
            (rect.x, rect.y),
            (rect.x + rect.width - diameter, rect.y),
            (rect.x, rect.y + rect.height - diameter),
            (
                rect.x + rect.width - diameter,
                rect.y + rect.height - diameter,
            ),
        ]
        for corner_x, corner_y in coordinates:
            shapes.Circle(
                corner_x + self.radius,
                corner_y + self.radius,
                self.radius,
                color=color,
            ).draw()

        # Draw rectangles to fill the gaps inside the rounded borders
        shapes.Rectangle(
            rect.x + self.radius,
            rect.y,
            rect.width - diameter,
            rect.height,
            color=color,
        ).draw()
        shapes.Rectangle(
            rect.x,
            rect.y + self.radius,
            rect.width,
            rect.height - diameter,
            color=color,
        ).draw()

        self.text.draw()

    def on_mouse_press(self, x, y, button, modifiers):
        if self.action and self.contains(x, y):
            self.toggled = not self.toggled
            self.action()
            return True



import math
import os
import re

import pyglet

from .widgets import Button, HBox, SimpleDropdown, Style, Rect
from .settings import FONT, WINDOW_MARGIN, UI_RATIO, LEGACY, PROG_NAME
from .displaywidget import GuiScreen
from .utils import sorted_resolutions, sorted_frequencies, find_matching_mode
from .utils import compute_bounding_box

hex_re = re.compile("^[0-9x]+$")


class UI(pyglet.window.Window):
    def __init__(self, width, height, displayInfo):
        super().__init__(width, height, PROG_NAME, resizable=True)
        self.selected_item = None
        self.scale_factor = 1
        but_w = 120
        but_h = 25
        self.cursor_coords = (0, 0)

        gui_screens: list[GuiScreen] = []
        self.gui_screens = gui_screens

        box = HBox(WINDOW_MARGIN, WINDOW_MARGIN, but_h)
        apply_but = Button(
            box.add(but_w),
            "Confirm",
            action=self.save_layout,
            style=Style(color=(120, 165, 240), bold=True),
        )
        self.resolutions = SimpleDropdown(
            box.add(but_w),
            "Resolution",
            [],
            onchange=self.update_screen_spec,
            # invert=True,
        )
        self.freqs = SimpleDropdown(
            box.add(but_w),
            "Rate",
            [],
            onchange=self._update_mode,
            # invert=True,
        )
        self.on_off_but = Button(
            box.add(but_w),
            "On/Off",
            action=self.toggle_screen_power,
            style=Style(highlight=(200, 100, 150), color=(100, 200, 150)),
            togglable=True,
        )

        # Loop over each screen in the displayInfo list
        for screen in displayInfo:
            # Get the position and mode width and height for this screen
            x, y = screen.position
            y = height - y

            max_width = max(m.width for m in screen.available)
            max_height = max(m.height for m in screen.available)

            if screen.mode:
                rect = Rect(
                    int(x / UI_RATIO),
                    int(y / UI_RATIO),
                    int(screen.mode.width / UI_RATIO),
                    int(screen.mode.height / UI_RATIO),
                )
            else:
                rect = Rect(
                    int(x / UI_RATIO),
                    int(y / UI_RATIO),
                    int(max_width / UI_RATIO),
                    int(max_height / UI_RATIO),
                )
            gs = GuiScreen(screen, rect)
            gs.genColor()
            gui_screens.append(gs)

        max_x = max(s.rect.right for s in gui_screens)
        min_x = min(s.rect.left for s in gui_screens)
        min_y = min(s.rect.top for s in gui_screens)
        max_y = max(s.rect.bottom for s in gui_screens)

        offsetX = (width - (max_x - min_x)) // 2
        offsetY = (height - (max_y - min_y)) // 2

        for screen in gui_screens:
            screen.set_position(screen.rect.x + offsetX, screen.rect.y + offsetY)
        self.center_layout(immediate=True)

        self.widgets: list[SimpleDropdown | Button] = [
            apply_but,
            self.on_off_but,
            self.resolutions,
            self.freqs,
        ]

    def on_mouse_motion(self, x, y, dx, dy):
        self.cursor_coords = (x, y)

    def on_mouse_press(self, x, y, button, modifiers):
        active_widget = None
        for wid in self.widgets:
            if wid.on_mouse_press(x, y, button, modifiers):
                active_widget = wid
                break
        else:
            for screen in self.gui_screens:
                if screen.rect.contains(x, y):
                    self.select_screen(screen)
                    break
            # else:
            # NOTE: enables screen unfocusing:
            #     self.selected_item = None

        for wid in self.widgets:
            if wid != active_widget:
                wid.unfocus()

    def select_screen(self, screen):
        self.selected_item = screen
        self.selected_item.dragging = True
        # make it last displayed + easy to find
        self.gui_screens.remove(screen)
        self.gui_screens.append(screen)

        cur_mode = screen.screen.mode
        # Update resolution
        res = sorted_resolutions(screen.screen.available)
        self.resolutions.options = [{"name": f"{r[0]}x{r[1]}", "value": r} for r in res]
        i = -1
        for i, r in enumerate(res):
            if r[0] == cur_mode.width and r[1] == cur_mode.height:
                break
        self.resolutions.selected_index = i
        self._update_frequencies(screen)

    def _update_frequencies(self, screen, mode=None):
        if mode is None:
            cur_mode = screen.screen.mode.width, screen.screen.mode.height
        else:
            cur_mode = mode
        freqs = sorted_frequencies(screen.screen.available, cur_mode[0], cur_mode[1])
        self.freqs.options = [{"name": f"{r:.2f} Hz", "value": r} for r in freqs]
        if mode is None:
            self.freqs.selected_index = freqs.index(screen.screen.mode.freq)
        else:
            self.freqs.selected_index = 0

    def on_mouse_drag(self, x, y, dx, dy, buttons, modifiers):
        if self.selected_item and self.selected_item.dragging:
            self.selected_item.set_position(
                self.selected_item.rect.x + dx, self.selected_item.rect.y + dy
            )

    def on_resize(self, width, height):
        pyglet.window.Window.on_resize(self, width, height)
        for wid in self.widgets:
            wid.rect.y = height - WINDOW_MARGIN - wid.rect.height
        self.center_layout(immediate=True)

    def on_mouse_release(self, x, y, button, modifiers):
        if self.selected_item and self.selected_item.dragging:
            self.snap_active_screen()
            self.selected_item.dragging = False
            self.center_layout()
        if self.selected_item:
            self.on_off_but.toggled = not self.selected_item.screen.active

    def on_draw(self):
        self.clear()
        # Draw a grey background
        pyglet.shapes.Rectangle(
            0, 0, self.width, self.height, color=(50, 50, 50, 255)
        ).draw()
        for screen in self.gui_screens:
            screen.highlighted = screen == self.selected_item
            screen.draw()
        if self.selected_item:
            for w in self.widgets:
                w.draw(self.cursor_coords)
        self.draw_status_label()

    def draw_status_label(self):
        status_label = pyglet.text.Label(
            self.get_status_text(),
            x=WINDOW_MARGIN,
            y=WINDOW_MARGIN,
            font_name=FONT,
            color=(200, 200, 200, 255),
        )
        status_label.draw()

    def center_layout(self, immediate=False):
        all_rects = [screen.target_rect for screen in self.gui_screens]
        avg_x, avg_y = Rect(*compute_bounding_box(all_rects)).center
        win_res = self.get_size()
        offX = (win_res[0] // 2) - avg_x
        offY = (win_res[1] // 2) - avg_y
        self.gui_screens = list(self.gui_screens)
        for screen in self.gui_screens:
            if immediate:
                screen.set_position(
                    screen.target_rect.x + offX, screen.target_rect.y + offY
                )
            else:
                screen.target_rect.x = screen.target_rect.x + offX
                screen.target_rect.y = screen.target_rect.y + offY

    def get_status_text(self):
        if self.selected_item:
            words = []
            for word in self.selected_item.screen.name.split():
                if not hex_re.match(word):
                    words.append(word)
            return " ".join(words)
        else:
            return "Select a monitor"

    def save_layout(self):
        screens = self.gui_screens
        min_x = UI_RATIO * min([screen.rect.x for screen in screens])
        min_y = UI_RATIO * min([-screen.rect.y for screen in screens])
        print("# Screens layout:")
        command = ["xrandr" if LEGACY else "wlr-randr"]
        for gs in screens:
            if not gs.screen.active:
                command.append(f"--output {gs.screen.uid} --off")
                continue
            x: int = (gs.rect.x * UI_RATIO) - min_x
            y: int = -gs.rect.y * UI_RATIO - min_y
            assert gs.screen.mode
            sep = "x" if LEGACY else ","
            uid = gs.screen.uid
            mode = f"{gs.screen.mode.width}x{gs.screen.mode.height}"
            command.append(f"--output {uid} --pos {x}{sep}{y} --mode {mode}")
        cmd = " ".join(command)
        print(cmd)
        os.system(cmd)

    def snap_active_screen(self):
        active_screen = self.gui_screens[-1]
        arect = active_screen.rect

        for wid in self.gui_screens[:-1]:
            if wid.rect.collide(active_screen.rect):
                # find the pair of corners
                # (one from gui_screen & one from active_screen)
                # which are closest
                other_screen_coords: list[tuple[int, int]] = [
                    wid.rect.topleft,
                    wid.rect.topright,
                    wid.rect.bottomright,
                    wid.rect.bottomleft,
                ]
                active_screen_coords: list[tuple[int, int]] = [
                    active_screen.rect.topleft,
                    active_screen.rect.topright,
                    active_screen.rect.bottomright,
                    active_screen.rect.bottomleft,
                ]

                def distance(point1: tuple[int, int], point2: tuple[int, int]):
                    a = (point1[0] - point2[0]) ** 2
                    b = (point1[1] - point2[1]) ** 2
                    return math.sqrt(a + b)

                # find which coordinates
                # from active_screen & gui_screen are closest
                min_distance = None
                closest_match = None
                for coord in active_screen_coords:
                    for other_screen_coord in other_screen_coords:
                        if (
                            min_distance is None
                            or distance(coord, other_screen_coord) < min_distance
                        ):
                            min_distance = distance(coord, other_screen_coord)
                            closest_match = other_screen_coord, coord
                assert closest_match is not None
                active_screen.target_rect.x -= closest_match[1][0] - closest_match[0][0]
                active_screen.target_rect.y -= closest_match[1][1] - closest_match[0][1]

    def toggle_screen_power(self):
        if self.selected_item:
            self.selected_item.screen.active = not self.selected_item.screen.active

    def update_screen_spec(self):
        self._update_frequencies(self.selected_item, self.resolutions.get_value())
        self._update_mode()

    def _update_mode(self):
        assert self.selected_item
        screen = self.selected_item.screen
        screen.mode = find_matching_mode(
            screen.available, self.resolutions.get_value(), self.freqs.get_value()
        )
        self.selected_item.target_rect.width = screen.mode.width // UI_RATIO
        self.selected_item.target_rect.height = screen.mode.height // UI_RATIO



import pyglet
from pyglet import shapes
from .utils import collidepoint
from .screens import Screen
from .widgets import Rect
import random


class GuiScreen:
    def __str__(self):
        return "<Screen %s (%s) - %s>" % (self.rect, self.color, self.screen.name)

    all_colors: tuple[tuple[int, int, int], ...] = (
        (172, 65, 66),
        (126, 141, 80),
        (229, 181, 102),
        (108, 153, 186),
        (158, 78, 133),
        (125, 213, 207),
        (208, 208, 208),
    )
    cur_color = 0

    def __init__(
        self,
        screen: Screen,
        rect: Rect,
        color: tuple[int, int, int] = (100, 100, 100),
    ):
        self.screen = screen
        self.rect = rect
        self.target_rect = rect.copy()
        self.color = color
        self.dragging = False
        self.highlighted = False

    def genColor(self):
        if self.cur_color >= len(self.all_colors):
            self.color = (
                random.randint(100, 200),
                random.randint(100, 200),
                random.randint(100, 200),
                255,
            )
        else:
            self.color = list(self.all_colors[self.cur_color]) + [255]
            GuiScreen.cur_color += 1
        self.drag_color = list(self.color)
        self.drag_color[-1] = 200

    @property
    def current_color(self):
        """The current_color property."""
        color = self.drag_color if self.dragging else self.color
        if self.screen.active:
            return color
        return [color[0] // 3, color[1] // 3, color[2] // 3, color[3]]

    @property
    def statusInfo(self):
        return "Screen identifier: " + self.screen.name

    def _animation_step(self, speed=1):
        r = self.rect
        t = self.target_rect
        for var in ("x", "y", "width", "height"):
            cv = getattr(r, var)
            tv = getattr(t, var)
            if cv != tv:
                if abs(tv - cv) <= 1:
                    setattr(r, var, tv)
                else:
                    if cv < tv:
                        setattr(r, var, min(tv, (cv + tv) / 2))
                    elif cv > tv:
                        setattr(r, var, max(tv, (cv + tv) / 2))

    def set_position(self, x, y):
        self.rect.x = x
        self.target_rect.x = x
        self.rect.y = y
        self.target_rect.y = y

    def draw(self):
        if self.rect != self.target_rect:
            self._animation_step()

        txt_color = (0, 0, 0, 200) if self.screen.active else (255, 255, 255, 120)
        border_color = (100, 100, 155)
        if not self.screen.active:
            border_color = (70, 70, 70)
        if self.highlighted:
            border_color = (255, 201, 0)
        # draw the background
        shapes.BorderedRectangle(
            self.rect.x,
            self.rect.y,
            self.rect.width,
            self.rect.height,
            border=9 if self.highlighted else 2,
            color=self.current_color,
            border_color=border_color,
        ).draw()
        # Render the screen uid as text
        tx, ty = self.rect.center
        pyglet.text.Label(
            self.screen.uid,
            anchor_x="center",
            anchor_y="center",
            x=tx,
            y=ty,
            font_size=16,
            color=(200, 200, 200, 255),
        ).draw()

        # Second caption line
        if self.screen.active:
            assert self.screen.mode
            label = "%dx%d" % (
                self.screen.mode.width,
                self.screen.mode.height,
            )
            pyglet.text.Label(
                label,
                anchor_x="center",
                anchor_y="center",
                x=tx,
                y=ty + 20,
                color=txt_color,
                bold=self.screen.active,
            ).draw()
            pyglet.text.Label(
                "%dHz" % self.screen.mode.freq,
                anchor_x="center",
                anchor_y="center",
                x=tx,
                y=ty - 20,
                color=txt_color,
                bold=self.screen.active,
            ).draw()



import os
import re
from typing import Tuple

__all__ = ["Mode", "Screen", "load", "LEGACY"]

LEGACY = not os.environ.get("WAYLAND_DISPLAY", False)
MODE_RE = re.compile(r"^(?P<width>\d+)x(?P<height>\d+)(?P<x>[+-]\d+)(?P<y>[+-]\d+)$")


class Mode:
    def __init__(self, width, height, freq):
        self.width = width
        self.height = height
        self.freq = freq

    def __repr__(self):
        return "%dx%d@ %.2f" % (self.width, self.height, self.freq)


class Screen:
    def __init__(
        self,
        uid: str,
        name: str,
        active: bool = False,
        position: Tuple[int, int] = (0, 0),
        mode: None | Mode = None,
    ):
        self.uid = uid
        self.name = name
        self.active = active
        self.position = position
        self.mode: Mode | None = mode
        self.available: list[Mode] = []

    def __repr__(self):
        return "<Screen%s %s [%s]>" % ("*" if self.active else "", self.name, self.mode)


displayInfo: list[Screen] = []


def load():
    import subprocess

    if displayInfo:
        displayInfo.clear()

    out = subprocess.getoutput("xrandr" if LEGACY else "wlr-randr")
    current_screen: None | Screen = None
    mode_mode = False
    for line in out.splitlines():
        if LEGACY and ("disconnected" in line or line.startswith("Screen")):
            continue
        if line[0] != " ":
            uid, name = line.split(None, 1)
            current_screen = Screen(uid=uid, name=name.strip('"'))
            try:
                chim = name.split("(", 1)[0].strip().rsplit(None, 1)[1]
            except IndexError:
                current_screen.active = False
            else:
                mode_re = MODE_RE.match(chim)
                if mode_re:
                    current_screen.position = (
                        int(mode_re.group("x")),
                        int(mode_re.group("y")),
                    )

            displayInfo.append(current_screen)
            mode_mode = False
        else:
            if line[2] != " ":
                mode_mode = False
            assert current_screen
            sline = line.strip()
            if LEGACY:
                res, freq = sline.split(None, 1)
                if not res.endswith("i"):
                    res = tuple(int(x) for x in res.split("x"))
                    current = "*" in freq
                    freq = freq.split(None, 1)[0].rstrip("*+")
                    current_screen.available.append(Mode(res[0], res[1], float(freq)))
                    if current:
                        current_screen.mode = current_screen.available[-1]
            else:
                if mode_mode:
                    res, freq = sline.split(",", 1)
                    res = res.split(None, 1)[0]
                    res = tuple(int(x) for x in res.split("x"))
                    freq, comment = freq.strip().split(None, 1)
                    current_screen.available.append(Mode(res[0], res[1], float(freq)))
                    if "current" in comment:
                        current_screen.mode = current_screen.available[-1]

                elif sline.startswith("Modes:"):
                    mode_mode = True
                elif sline.startswith("Enabled"):
                    current_screen.active = "yes" in sline
                elif sline.startswith("Position"):
                    current_screen.position = tuple(
                        int(x) for x in sline.split(":")[1].strip().split(",")
                    )



def brighten(color):
    return [min(255, c + 20) for c in color]


def collidepoint(x, y, x2, y2, xp, yp):
    assert x < x2
    assert y < y2
    return x < xp < x2 and y < yp < y2


def sorted_resolutions(modes):
    res = set((m.width, m.height) for m in modes)
    lres = list(res)
    lres.sort(reverse=True)
    return lres


def sorted_frequencies(modes, filter_w=None, filter_h=None):
    filtered_modes = modes.copy()
    if filter_w:
        filtered_modes = filter(lambda m: m.width == filter_w, filtered_modes)
    if filter_h:
        filtered_modes = filter(lambda m: m.height == filter_h, filtered_modes)
    res = set(m.freq for m in filtered_modes)
    lres = list(res)
    lres.sort(reverse=True)
    return lres


def find_matching_mode(modes, res, freq):
    for mode in modes:
        if mode.width == res[0] and mode.height == res[1] and mode.freq == freq:
            return mode


def compute_bounding_box(rects):
    min_x = min(r.x for r in rects)
    min_y = min(r.y for r in rects)
    max_x = max(r.x + r.width for r in rects)
    max_y = max(r.y + r.height for r in rects)
    return (min_x, min_y, max_x - min_x, max_y - min_y)



import pyglet
from .gui import UI
from .settings import UI_RATIO, PROG_NAME
from .screens import displayInfo, load

try:
    import setproctitle

    setproctitle.setproctitle(PROG_NAME)
except ImportError:
    pass


def main():
    load()
    max_width = int(
        sum(
            max(screen.available, key=lambda mode: mode.width).width
            for screen in displayInfo
        )
        // UI_RATIO
    )
    max_height = int(
        sum(
            max(screen.available, key=lambda mode: mode.height).height
            for screen in displayInfo
        )
        // UI_RATIO
    )
    average_width = int(
        sum(
            max(screen.available, key=lambda mode: mode.width).width
            for screen in displayInfo
        )
        / len(displayInfo)
        // UI_RATIO
    )
    average_height = int(
        sum(
            max(screen.available, key=lambda mode: mode.height).height
            for screen in displayInfo
        )
        / len(displayInfo)
        // UI_RATIO
    )

    width = max_width + average_width * 2
    height = max_height + average_height * 2
    window = UI(width, height, displayInfo)
    window.set_wm_class(PROG_NAME)
    pyglet.app.run()
